<?php
/**
 * @version		$Id: installer.php 4.0 2021-10-22
 * @package		Joomla
 * @subpackage	Content
 * @copyright	Copyright (C) 2013 dejure.org. All rights reserved.
 * @license		This program is free software: you can redistribute it and/or modify
 *				it under the terms of the GNU General Public License as published by
 *				the Free Software Foundation, either version 3 of the License, or
 *				(at your option) any later version.
 *
 *				This program is distributed in the hope that it will be useful,
 *				but WITHOUT ANY WARRANTY; without even the implied warranty of
 *				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *				GNU General Public License for more details.
 *
 *				You should have received a copy of the GNU General Public License
 *				along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Use specific namespaces to make code more readable
use Joomla\CMS\Installer\InstallerScript;
use Joomla\CMS\Installer\Adapter\PluginAdapter;
use Joomla\CMS\Factory;

// No direct access.
defined('_JEXEC') or die;

class plgContentVernetzungInstallerScript extends InstallerScript
{
	private $djo_table = 'djo_vernetzung_cache';
	private $db = null;

	/**
	 * Constructor
	 */
	public function __construct() {
		$this->db = Factory::getDbo();
		$query = "DROP TABLE IF EXISTS ".$this->db->quoteName($this->djo_table);
		$this->db->setQuery($query);
		$this->db->execute();
	}

	/**
	 * Called before any type of action
	 *
	 * @param   string         $type    Which action is happening (install|uninstall|discover_install)
	 * @param   PluginAdapter  $parent  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	//public function preflight($type, PluginAdapter $parent);

	/**
	 * Called after any type of action
	 *
	 * @param   string         $type    Which action is happening (install|uninstall|discover_install)
	 * @param   PluginAdapter  $parent  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	//public function postflight($type, PluginAdapter $parent);

	/**
	 * Called on installation
	 *
	 * @param   PluginAdapter  $parent  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function install(PluginAdapter $parent) {
		$query = "CREATE TABLE ".$this->db->quoteName($this->djo_table)." (
			schluessel varchar(32),
			text mediumtext NOT NULL,
			gespeichert datetime NOT NULL default '0000-00-00 00:00:00',
			UNIQUE KEY ID (schluessel)
		);";
		$this->db->setQuery($query);
		return $this->db->execute();
	}

	/**
	 * Called on update
	 *
	 * @param   PluginAdapter  $parent  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function update(PluginAdapter $parent) {
		$query = "DROP TABLE IF EXISTS TABLE ".$this->db->quoteName($this->djo_table).";
		CREATE TABLE ".$this->db->quoteName($this->djo_table)." (
			schluessel varchar(32),
			text mediumtext NOT NULL,
			gespeichert datetime NOT NULL default '0000-00-00 00:00:00',
			UNIQUE KEY ID (schluessel)
		);";
		$this->db->setQuery($query);
		return $this->db->execute();
	}

	/**
	 * Called on uninstallation
	 *
	 * @param   PluginAdapter  $parent  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function uninstall(PluginAdapter $parent) {
		$query = "DROP TABLE IF EXISTS ".$this->db->quoteName($this->djo_table);
		$this->db->setQuery($query);
		return $this->db->execute();
	}
}

?>
