<?php
/**
 * @version		$Id: vernetzung.php 4.0 2021-10-22 08:18:00Z $
 * @package		Joomla.Plugin
 * @subpackage	Content
 * @copyright	Copyright (C) 2014 dejure.org GmbH All rights reserved.
 * @license		This program is free software: you can redistribute it and/or modify
 *				it under the terms of the GNU General Public License as published by
 *				the Free Software Foundation, either version 3 of the License, or
 *				(at your option) any later version.
 *
 *				This program is distributed in the hope that it will be useful,
 *				but WITHOUT ANY WARRANTY; without even the implied warranty of
 *				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *				GNU General Public License for more details.
 *
 *				You should have received a copy of the GNU General Public License
 *				along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Use specific namespaces to make code more readable
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\String\StringHelper;

// No direct access.
defined('_JEXEC') or die;

define('DJO_VERSION', '4.0');

class plgContentVernetzung extends CMSPlugin
{
	// Inherited from CMSPlugin
	protected $db;
	protected $app;

	private $djo_table = 'djo_vernetzung_cache';
	private $md5_params = null;

	function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
		$this->md5_params = md5($this->params->get('target').$this->params->get('class').$this->params->get('linkstil').$this->params->get('buzer'));
	}

	public function onContentPrepare($context, &$article, &$params, $page = 0) {
		$article->text = $this->djo_vernetzen($article->text);
		return true;
	}

	private function djo_vernetzen(&$ausgabetext) {
		if (StringHelper::strpos($ausgabetext, '{vernetzen=off}') !== false) {
			$ausgabetext = StringHelper::str_ireplace('{vernetzen=off}', '', $ausgabetext);
			return $ausgabetext;
		}
		if (!preg_match("/§|&sect;|Art\.|\/[0-9][0-9](?![0-9\/])| [0-9][0-9]?[\/\.][0-9][0-9](?![0-9\.])|[0-9], [0-9]/", $ausgabetext)) {
			return $ausgabetext;
		}
		$params['anbieterkennung'] = $this->app->get('sitename');
		$params['kontaktemail'] = 'webmaster@'.$_SERVER['HTTP_HOST']; # Wird im Rahmen der Serverkommunikation jeweils an dejure.org uebertragen und dient im uebrigen ausschliesslich der Kommunikation bei technischen Problemen.
		$params['target'] = $this->params->get('target'); # Zum Beispiel fuer oeffnen in neuem Fenster: "_blank"
		$params['class'] = $this->params->get('class');
		$params['linkstil'] = $this->params->get('linkstil');
		$params['buzer'] = $this->params->get('buzer');
		$ausgabetext = trim($ausgabetext);
		$rueckgabe = $this->vernetzen_ueber_cache($ausgabetext);
		if (empty($rueckgabe)) {
			$rueckgabe = $this->vernetzen_ueber_dejure_org($ausgabetext, array("Anbieterkennung" => $params['anbieterkennung']."__".$params['kontaktemail'], "target" => $params['target'], "class" => $params['class'], 'version' => 'joomla-'.DJO_VERSION, 'format' => $params['linkstil'], 'buzer' => $params['buzer']));
		}
		if (date('G') < 6) { // Zwischen 0 und 6 Uhr alle alten Eintraege im Cache loeschen, die aelter als 4 Tage sind. Dies loest regelmaessig eine aktualisierende Neuvernetzung aus
			$this->cache_verfall();
		}
		return $rueckgabe;
	}

	private function vernetzen_ueber_cache($ausgangstext) {
		$schluessel = strlen($ausgangstext) . '-' . substr(md5($this->md5_params.$ausgangstext), 0, -1-strlen(strlen($ausgangstext)));;
		$query = "
			SELECT ".$this->db->quoteName('text')."
			FROM ".$this->db->quoteName($this->djo_table)."
			WHERE ".$this->db->quoteName('schluessel')." = ".$this->db->quote($schluessel).";
		";
		$this->db->setQuery($query);
		$rueckgabe = $this->db->loadResult();
		if (empty($rueckgabe) || $rueckgabe == '<!-- idem -->') {
			return '';
		} else {
			return $rueckgabe;
		}
	}

	private function ablegen_in_cache($ausgangstext, $rueckgabe) {
		$schluessel = strlen($ausgangstext) . '-' . substr(md5($this->md5_params.$ausgangstext), 0, -1-strlen(strlen($ausgangstext)));
		if ($ausgangstext == $rueckgabe) {
			$text = '<!-- idem -->';
		} else {
			$text = $rueckgabe;
		}
		$query = "
			REPLACE INTO ".$this->db->quoteName($this->djo_table)."
			(".$this->db->quoteName('schluessel').", ".$this->db->quoteName('text').", ".$this->db->quoteName('gespeichert').")
			VALUES (".$this->db->quote($schluessel).", ".$this->db->quote($text).", NOW());
		";
		$this->db->setQuery($query);
		$this->db->execute();
	}

	private function cache_verfall($stunden = null) {
		if (rand(0, 25) < 1) {
			$query = "
				DELETE FROM ".$this->db->quoteName($this->djo_table)."
				WHERE ".$this->db->quoteName('gespeichert')." < DATE_SUB(NOW(), INTERVAL 4 DAY)
			;";
			$this->db->setQuery($query);
			$this->db->execute();
		}
	}

	private function integritaetskontrolle_und_cache($ausgangstext, $neuertext) {
		if (preg_replace("/<a href=\"http:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $ausgangstext) == preg_replace("/<a href=\"http:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $neuertext)) {
			$this->ablegen_in_cache($ausgangstext, $neuertext);
			return $neuertext;
		}
		else {
			return $ausgangstext;
		}
	}

	private function vernetzen_ueber_dejure_org($ausgangstext, $parameter = array()) {
		$uebergabe = "Originaltext=".urlencode($ausgangstext);
		foreach ($parameter as $option => $wert) {
			if ($option == "zeitlimit_in_sekunden") {
				$zeitlimit_in_sekunden = $wert;
			}
			else {
				$uebergabe .= "&".urlencode($option)."=".urlencode($wert);
			}
		}
		if (!isset($zeitlimit_in_sekunden)) {
			$zeitlimit_in_sekunden = 3;
		}
		$uri = JUri::getInstance();
		$hostname = $uri->getHost();

		$header = "POST http://rechtsnetz.dejure.org/dienste/vernetzung/vernetzen HTTP/1.0\r\n";
		$header .= 'User-Agent: '.$hostname.' (Joomla-Vernetzung '.DJO_VERSION.')'."\r\n";
		$header .= 'Content-type: application/x-www-form-urlencoded'."\r\n";
		$header .= 'Content-length: '.strlen($uebergabe)."\r\n";
		$header .= 'Host: rechtsnetz.dejure.org'."\r\n";
		$header .= 'Connection: close'."\r\n";
		$header .= "\r\n";

		$fp = @fsockopen("rechtsnetz.dejure.org", 80, $errno, $errstr, $zeitlimit_in_sekunden);

		if (!$fp) { // Verbindung gescheitert
			return $ausgangstext;
		}
		else {
			socket_set_timeout($fp, $zeitlimit_in_sekunden, 0); // Socket nach $zeitlimit_in_sekunden Sekunden auf jeden Fall wieder frei geben
			socket_set_blocking($fp, true);
			fputs($fp, $header.$uebergabe);
			$timeOutSock = false;
			$eofSock = false;
			$rueckgabe = '';
			while (!$eofSock && !$timeOutSock) {
				$rueckgabe.= fgets($fp, 1024); //
				$stSock = socket_get_status($fp);
				$eofSock = $stSock["eof"];
				$timeOutSock = $stSock["timed_out"];
			}
			fclose($fp);
			if (!preg_match("/^(.*?)\r?\n\r?\n\r?\n?(.*)/s",$rueckgabe, $rueckgabeARR)) {
				return $ausgangstext; // Zeitueberschreitung oder Verbindungsproblem
			}
			else if (strpos($rueckgabeARR[1],"200 OK")===false) {
				return $ausgangstext; // sonstiges Serverproblem
			}
			else {
				$rueckgabe = $rueckgabeARR[2];
				if (strlen($rueckgabe) < strlen($ausgangstext)) {
					return $ausgangstext;
				}
				return $this->integritaetskontrolle_und_cache($ausgangstext, $rueckgabe);
			}
		}
	}
}
